/*
 * nct6796d.cc
 * Copyright (C) 2025 Henry Hu
 *
 * Distributed under terms of the 3-clause BSD license.
 */

#include "nuvoton_chip_info.h"

namespace bsdsensors {

std::map<NuvotonTempSource, uint8_t> kNCT6796DTempSource{
        {NuvotonTempSource::kSourceSYSTIN, 1},
        {NuvotonTempSource::kSourceCPUTIN, 2},
        {NuvotonTempSource::kSourceAUXTIN0, 3},
        {NuvotonTempSource::kSourceAUXTIN1, 4},
        {NuvotonTempSource::kSourceAUXTIN2, 5},
        {NuvotonTempSource::kSourceAUXTIN3, 6},
        {NuvotonTempSource::kSourceAUXTIN4, 7},
        {NuvotonTempSource::kSourceSMBUSMASTER0, 8},
        {NuvotonTempSource::kSourceSMBUSMASTER1, 9},
        {NuvotonTempSource::kSourcePECI0, 16},
        {NuvotonTempSource::kSourcePECI1, 17},
        {NuvotonTempSource::kSourcePCHCPUMAX, 18},
        {NuvotonTempSource::kSourcePCH, 19},
        {NuvotonTempSource::kSourcePCHCPU, 20},
        {NuvotonTempSource::kSourcePCHMCH, 21},
        {NuvotonTempSource::kSourceDIM0, 22},
        {NuvotonTempSource::kSourceDIM1, 23},
        {NuvotonTempSource::kSourceDIM2, 24},
        {NuvotonTempSource::kSourceDIM3, 25},
        {NuvotonTempSource::kSourceBYTE0, 26},
        {NuvotonTempSource::kSourceBYTE1, 27},
        {NuvotonTempSource::kSourcePECI0Calib, 28},
        {NuvotonTempSource::kSourcePECI1Calib, 29},
        {NuvotonTempSource::kSourceVirtualTemp1, 31},
};

const NuvotonChipInfo kNCT6796D = {
    0xD42B,
    "NCT6796D",
    /*vendor_id_addr=*/{0, 0x4F},
    // Fan speed info
    {
        {
            .name = "SYSFAN",
            .rpm_high = {4, 0xC0},
            .rpm_low = {4, 0xC1},
            .count = {4, 0xB0, /*bits=*/{}, /*next=*/{4, 0xB1, /*bits=*/{4, 0}}},
            .control = NuvotonFanControlInfo{
                .mode_select = {1, 0x02, {7, 4}},
                .output_value_write = {1, 0x09},
                .output_value_write_dc = {1, 0x09, {7, 2}},
                .support_dc = true,
                .output_type = {0, 0x24, {4}},
                .output_value_read = {0, 0x01},
                .temp_source = {1, 0x00, {4, 0}},
                .temp_value_int = {0, 0x73},
                .temp_value_frac = {0, 0x74, {7}},
                .smart_fan = {.available = true,
                    .thermal_cruise = {
                        .target_temp = {1, 0x01},
                        .tolerance = {1, 0x02, {2, 0}},
                        .start_value = {1, 0x06},
                        .stop_value = {1, 0x05},
                        .keep_min_output = {1, 0x00, {7}},
                        .stop_time = {1, 0x07},
                        .step_up_time = {1, 0x03},
                        .step_down_time = {1, 0x04},
                        .critical_temp = {1, 0x35},
                    },
                    .speed_cruise = {
                        .target_speed_count_low = {1, 0x01},
                        .target_speed_count_high = {1, 0x0C, {3, 0}},
                        .tolerance_low = {1, 0x02, {2, 0}},
                        .tolerance_high = {1, 0x0C, {6, 4}},
                        .step_up_time = {1, 0x03},
                        .step_down_time = {1, 0x04},
                        .step_up_value = {1, 0x66, {7, 4}},
                        .step_down_value = {1, 0x66, {3, 0}},
                    },
                },
                .smart_fan4 = {.available = true,
                    .control_points = {
                        {{1, 0x21}, {1, 0x27}},
                        {{1, 0x22}, {1, 0x28}},
                        {{1, 0x23}, {1, 0x29}},
                        {{1, 0x24}, {1, 0x2A}},
                    },
                },
            },
        },
        {
            .name = "CPUFAN",
            .rpm_high = {4, 0xC2},
            .rpm_low = {4, 0xC3},
            .count = {4, 0xB2, /*bits=*/{}, /*next=*/{4, 0xB3, /*bits=*/{4, 0}}},
            .control = NuvotonFanControlInfo{
                .mode_select = {2, 0x02, {7, 4}},
                .output_value_write = {2, 0x09},
                .output_value_write_dc = {},
                .support_dc = false,
                .output_type = {0, 0x24, {3}},
                .output_value_read = {0, 0x03},
                .temp_source = {2, 0x00, {4, 0}},
                .temp_value_int = {0, 0x75},
                .temp_value_frac = {0, 0x76, {7}},
                .smart_fan = {.available = true,
                    .thermal_cruise = {
                        .target_temp = {2, 0x01},
                        .tolerance = {2, 0x02, {2, 0}},
                        .start_value = {2, 0x06},
                        .stop_value = {2, 0x05},
                        .keep_min_output = {2, 0x00, {7}},
                        .stop_time = {2, 0x07},
                        .step_up_time = {2, 0x03},
                        .step_down_time = {2, 0x04},
                        .critical_temp = {2, 0x35},
                    },
                    .speed_cruise = {
                        .target_speed_count_low = {2, 0x01},
                        .target_speed_count_high = {2, 0x0C, {3, 0}},
                        .tolerance_low = {2, 0x02, {2, 0}},
                        .tolerance_high = {2, 0x0C, {6, 4}},
                        .step_up_time = {2, 0x03},
                        .step_down_time = {2, 0x04},
                        .step_up_value = {2, 0x66, {7, 4}},
                        .step_down_value = {2, 0x66, {3, 0}},
                    },
                },
                .smart_fan4 = {.available = true,
                    .control_points = {
                        {{2, 0x21}, {2, 0x27}},
                        {{2, 0x22}, {2, 0x28}},
                        {{2, 0x23}, {2, 0x29}},
                        {{2, 0x24}, {2, 0x2A}},
                    },
                },
            },
        },
        {
            .name = "AUXFAN0",
            .rpm_high = {4, 0xC4},
            .rpm_low = {4, 0xC5},
            .count = {4, 0xB4, /*bits=*/{}, /*next=*/{4, 0xB5, /*bits=*/{4, 0}}},
            .control = NuvotonFanControlInfo{
                .mode_select = {3, 0x02, {7, 4}},
                .output_value_write = {3, 0x09},
                .output_value_write_dc = {},
                .support_dc = false,
                .output_type = {0, 0x24, {5}},
                .output_value_read = {0, 0x11},
                .temp_source = {3, 0x00, {4, 0}},
                .temp_value_int = {0, 0x77},
                .temp_value_frac = {0, 0x78, {7}},
                .smart_fan = {.available = true,
                    .thermal_cruise = {
                        .target_temp = {3, 0x01},
                        .tolerance = {3, 0x02, {2, 0}},
                        .start_value = {3, 0x06},
                        .stop_value = {3, 0x05},
                        .keep_min_output = {3, 0x00, {7}},
                        .stop_time = {3, 0x07},
                        .step_up_time = {3, 0x03},
                        .step_down_time = {3, 0x04},
                        .critical_temp = {3, 0x35},
                    },
                    .speed_cruise = {
                        .target_speed_count_low = {3, 0x01},
                        .target_speed_count_high = {3, 0x0C, {3, 0}},
                        .tolerance_low = {3, 0x02, {2, 0}},
                        .tolerance_high = {3, 0x0C, {6, 4}},
                        .step_up_time = {3, 0x03},
                        .step_down_time = {3, 0x04},
                        .step_up_value = {3, 0x66, {7, 4}},
                        .step_down_value = {3, 0x66, {3, 0}},
                    },
                },
                .smart_fan4 = {.available = true,
                    .control_points = {
                        {{3, 0x21}, {3, 0x27}},
                        {{3, 0x22}, {3, 0x28}},
                        {{3, 0x23}, {3, 0x29}},
                        {{3, 0x24}, {3, 0x2A}},
                    },
                },
            },
        },
        {
            .name = "AUXFAN1",
            .rpm_high = {4, 0xC6},
            .rpm_low = {4, 0xC7},
            .count = {4, 0xB6, /*bits=*/{}, /*next=*/{4, 0xB7, /*bits=*/{4, 0}}},
            .control = NuvotonFanControlInfo{
                .mode_select = {8, 0x02, {7, 4}},
                .output_value_write = {8, 0x09},
                .output_value_write_dc = {},
                .support_dc = false,
                .output_type = {0, 0x24, {6}},
                .output_value_read = {0, 0x13},
                .temp_source = {8, 0x00, {4, 0}},
                .temp_value_int = {0, 0x79},
                .temp_value_frac = {0, 0x7A, {7}},
                .smart_fan = {.available = true,
                    .thermal_cruise = {
                        .target_temp = {8, 0x01},
                        .tolerance = {8, 0x02, {2, 0}},
                        .start_value = {8, 0x06},
                        .stop_value = {8, 0x05},
                        .keep_min_output = {8, 0x00, {7}},
                        .stop_time = {8, 0x07},
                        .step_up_time = {8, 0x03},
                        .step_down_time = {8, 0x04},
                        .critical_temp = {8, 0x35},
                    },
                    .speed_cruise = {
                        .target_speed_count_low = {8, 0x01},
                        .target_speed_count_high = {8, 0x0C, {3, 0}},
                        .tolerance_low = {8, 0x02, {2, 0}},
                        .tolerance_high = {8, 0x0C, {6, 4}},
                        .step_up_time = {8, 0x03},
                        .step_down_time = {8, 0x04},
                        .step_up_value = {8, 0x66, {7, 4}},
                        .step_down_value = {8, 0x66, {3, 0}},
                    },
                },
                .smart_fan4 = {.available = true,
                    .control_points = {
                        {{8, 0x21}, {8, 0x27}},
                        {{8, 0x22}, {8, 0x28}},
                        {{8, 0x23}, {8, 0x29}},
                        {{8, 0x24}, {8, 0x2A}},
                    },
                },
            },
        },
        {
            .name = "AUXFAN2",
            .rpm_high = {4, 0xC8},
            .rpm_low = {4, 0xC9},
            .count = {4, 0xB8, /*bits=*/{}, /*next=*/{4, 0xB9, /*bits=*/{4, 0}}},
            .control = NuvotonFanControlInfo{
                .mode_select = {9, 0x02, {7, 4}},
                .output_value_write = {9, 0x09},
                .output_value_write_dc = {},
                .support_dc = false,
                .output_type = {0, 0x24, {7}},
                .output_value_read = {0, 0x15},
                .temp_source = {9, 0x00, {4, 0}},
                .temp_value_int = {0, 0x7B},
                .temp_value_frac = {0, 0x7C, {7}},
                .smart_fan = {.available = true,
                    .thermal_cruise = {
                        .target_temp = {9, 0x01},
                        .tolerance = {9, 0x02, {2, 0}},
                        .start_value = {9, 0x06},
                        .stop_value = {9, 0x05},
                        .keep_min_output = {9, 0x00, {7}},
                        .stop_time = {9, 0x07},
                        .step_up_time = {9, 0x03},
                        .step_down_time = {9, 0x04},
                        .critical_temp = {9, 0x35},
                    },
                    .speed_cruise = {
                        .target_speed_count_low = {9, 0x01},
                        .target_speed_count_high = {9, 0x0C, {3, 0}},
                        .tolerance_low = {9, 0x02, {2, 0}},
                        .tolerance_high = {9, 0x0C, {6, 4}},
                        .step_up_time = {9, 0x03},
                        .step_down_time = {9, 0x04},
                        .step_up_value = {9, 0x66, {7, 4}},
                        .step_down_value = {9, 0x66, {3, 0}},
                    },
                },
                .smart_fan4 = {.available = true,
                    .control_points = {
                        {{9, 0x21}, {9, 0x27}},
                        {{9, 0x22}, {9, 0x28}},
                        {{9, 0x23}, {9, 0x29}},
                        {{9, 0x24}, {9, 0x2A}},
                    },
                },
            },
        },
        {
            .name = "AUXFAN3",
            .rpm_high = {4, 0xCA},
            .rpm_low = {4, 0xCB},
            .count = {4, 0xBA, /*bits=*/{}, /*next=*/{4, 0xBB, /*bits=*/{4, 0}}},
            .control = NuvotonFanControlInfo{
                .mode_select = {0xA, 0x02, {7, 4}},
                .output_value_write = {0xA, 0x09},
                .output_value_write_dc = {},
                .support_dc = false,
                .output_type = {0, 0x24, {1}},
                .output_value_read = {0, 0x17},
                .temp_source = {0xA, 0x00, {4, 0}},
                .temp_value_int = {0, 0x7D},
                .temp_value_frac = {0, 0x7E, {7}},
                .smart_fan = {.available = true,
                    .thermal_cruise = {
                        .target_temp = {0xA, 0x01},
                        .tolerance = {0xA, 0x02, {2, 0}},
                        .start_value = {0xA, 0x06},
                        .stop_value = {0xA, 0x05},
                        .keep_min_output = {0xA, 0x00, {7}},
                        .stop_time = {0xA, 0x07},
                        .step_up_time = {0xA, 0x03},
                        .step_down_time = {0xA, 0x04},
                        .critical_temp = {0xA, 0x35},
                    },
                    .speed_cruise = {
                        .target_speed_count_low = {0xA, 0x01},
                        .target_speed_count_high = {0xA, 0x0C, {3, 0}},
                        .tolerance_low = {0xA, 0x02, {2, 0}},
                        .tolerance_high = {0xA, 0x0C, {6, 4}},
                        .step_up_time = {0xA, 0x03},
                        .step_down_time = {0xA, 0x04},
                        .step_up_value = {0xA, 0x66, {7, 4}},
                        .step_down_value = {0xA, 0x66, {3, 0}},
                    },
                },
                .smart_fan4 = {.available = true,
                    .control_points = {
                        {{0xA, 0x21}, {0xA, 0x27}},
                        {{0xA, 0x22}, {0xA, 0x28}},
                        {{0xA, 0x23}, {0xA, 0x29}},
                        {{0xA, 0x24}, {0xA, 0x2A}},
                    },
                },
            },
        },
        {
            .name = "AUXFAN4",
            .rpm_high = {4, 0xCE},
            .rpm_low = {4, 0xCF},
            .count = {4, 0xCC, /*bits=*/{}, /*next=*/{4, 0xCD, /*bits=*/{4, 0}}},
            .control = NuvotonFanControlInfo{
                .mode_select = {0xB, 0x02, {7, 4}},
                .output_value_write = {0xB, 0x09},
                .output_value_write_dc = {},
                .support_dc = false,
                .output_type = {0, 0x25, {0}},
                .output_value_read = {0, 0x29},
                .temp_source = {0xB, 0x00, {4, 0}},
                .temp_value_int = {4, 0xA0},
                .temp_value_frac = {4, 0x9E, {6}},
                .smart_fan = {.available = true,
                    .thermal_cruise = {
                        .target_temp = {0xB, 0x01},
                        .tolerance = {0xB, 0x02, {2, 0}},
                        .start_value = {0xB, 0x06},
                        .stop_value = {0xB, 0x05},
                        .keep_min_output = {0xB, 0x00, {7}},
                        .stop_time = {0xB, 0x07},
                        .step_up_time = {0xB, 0x03},
                        .step_down_time = {0xB, 0x04},
                        .critical_temp = {0xB, 0x35},
                    },
                    .speed_cruise = {
                        .target_speed_count_low = {0xB, 0x01},
                        .target_speed_count_high = {0xB, 0x0C, {3, 0}},
                        .tolerance_low = {0xB, 0x02, {2, 0}},
                        .tolerance_high = {0xB, 0x0C, {6, 4}},
                        .step_up_time = {0xB, 0x03},
                        .step_down_time = {0xB, 0x04},
                        .step_up_value = {0xB, 0x66, {7, 4}},
                        .step_down_value = {0xB, 0x66, {3, 0}},
                    },
                },
                .smart_fan4 = {.available = true,
                    .control_points = {
                        {{0xB, 0x21}, {0xB, 0x27}},
                        {{0xB, 0x22}, {0xB, 0x28}},
                        {{0xB, 0x23}, {0xB, 0x29}},
                        {{0xB, 0x24}, {0xB, 0x2A}},
                    },
                },
            },
        },
    },
    // Temp sensors
    kNCT6796DTempSource,
    {
        {"SMIOVT1", /*val_int=*/{0, 0x27}, /*val_frac=*/{},
         /*has_peci_frac=*/false, /*can_select=*/true,
         /*select=*/{6, 0x21, {4, 0}}},
        {"SMIOVT2", /*val_int=*/{1, 0x50}, /*val_frac=*/{},
         /*has_peci_frac=*/false, /*can_select=*/true,
         /*select=*/{6, 0x22, {4, 0}}},
        {"SMIOVT3", /*val_int=*/{6, 0x70}, /*val_frac=*/{},
         /*has_peci_frac=*/false, /*can_select=*/true,
         /*select=*/{0xC, 0x26, {4, 0}}},
        {"SMIOVT4", /*val_int=*/{6, 0x72}, /*val_frac=*/{},
         /*has_peci_frac=*/false, /*can_select=*/true,
         /*select=*/{0xC, 0x27, {4, 0}}},
        {"SMIOVT5", /*val_int=*/{6, 0x74}, /*val_frac=*/{},
         /*has_peci_frac=*/false, /*can_select=*/true,
         /*select=*/{0xC, 0x28, {4, 0}}},
        {"SMIOVT6", /*val_int=*/{6, 0x76}, /*val_frac=*/{},
         /*has_peci_frac=*/false, /*can_select=*/true,
         /*select=*/{0xC, 0x29, {4, 0}}},
        {"SMIOVT7", /*val_int=*/{6, 0x78}, /*val_frac=*/{},
         /*has_peci_frac=*/false, /*can_select=*/true,
         /*select=*/{0xC, 0x2A, {4, 0}}},
        {"SMIOVT8", /*val_int=*/{6, 0x7A}, /*val_frac=*/{},
         /*has_peci_frac=*/false, /*can_select=*/true,
         /*select=*/{0xC, 0x2B, {4, 0}}},
        {"SYSTIN", {4, 0x90}},
        {"CPUTIN", {4, 0x91}},
        {"AUXTIN0", {4, 0x92}},
        {"AUXTIN1", {4, 0x93}},
        {"AUXTIN2", {4, 0x94}},
        {"AUXTIN3", {4, 0x95}},
        {"PECI0Calibration", {4, 0xF4}},
        {"PECI1Calibration", {4, 0xF5}},
        {"PCH_CPU_MAX", {4, 0x00}},
        {"PCH", {4, 0x01}},
        {"PCH_CPU", {4, 0x02}, {4, 0x03, {7, 2}}, true},
        {"MCH", {4, 0x04}},
        {"DIM0", {4, 0x05}},
        {"DIM1", {4, 0x06}},
        {"DIM2", {4, 0x07}},
        {"DIM3", {4, 0x08}},
        {"TSI0", {4, 0x09}, {4, 0x0A, {7, 5}}},
        {"TSI1", {4, 0x0B}, {4, 0x0C, {7, 5}}},
        {"BYTE0", {4, 0x19}},
        {"BYTE1", {4, 0x1A}},
    },
    // Volts
    {
        {"Vcore", {4, 0x80}},
        {"Vin1", {4, 0x81}},
        {"AVsb", {4, 0x82}, 2.0},
        {"3Vcc", {4, 0x83}, 2.0},
        {"Vin0", {4, 0x84}},
        {"Vin8", {4, 0x85}},
        {"Vin4", {4, 0x86}},
        {"3Vsb", {4, 0x87}, 2.0},
        {"Vbat", {4, 0x88}},
        {"Vtt", {4, 0x89}},
        {"Vin5", {4, 0x8A}},
        {"Vin6", {4, 0x8B}},
        {"Vin2", {4, 0x8C}},
        {"Vin3", {4, 0x8D}},
        {"Vin7", {4, 0x8E}},
        {"Vin9", {4, 0x8F}},
    }
};

RegisterChip<NuvotonChipInfo> kRegisterNCT6796D(kNCT6796D);

}
